unit SUtilBase;

interface

uses Windows;

type
  _TSFile = class(TObject)
  private
    FHandle: THandle;
    FName: string;
    function GetSize: Cardinal;
  public
    constructor Create(const FileName: string; const Mode: string);
    destructor Destroy; override;
    function Read(var Buffer; Count: Integer): Cardinal;
    function Write(var Buffer; Count: Integer): Cardinal;
    function Seek(Offset: Integer): Cardinal;
    property Size: Cardinal read GetSize;
    property Name: string read FName;
  end;

  _TListDir = class(TObject)
  private
    FHandle: THandle;
    FResult: Boolean;
    FFindData: TWin32FindData;
    FIsDir: Boolean;
  public
    constructor Create(const Path: string);
    constructor CreateWithGlob(const Path: string; Glob: string);
    destructor Destroy; override;
    function Each(var FileName: string): Boolean;
    property IsDir: Boolean read FIsDir;
  end;

implementation

uses SUtil;

{ _TSFile }

constructor _TSFile.Create(const FileName: string; const Mode: string);
var
  access, sharemode, creation: DWORD;
begin
  sharemode := FILE_SHARE_READ or FILE_SHARE_WRITE;
  if Mode = 'w' then
  begin
    access := GENERIC_WRITE;
    creation := CREATE_ALWAYS;
    if FileName = '' then
    begin
      FName := '<STDOUT>';
      FHandle := GetStdHandle(STD_OUTPUT_HANDLE);
    end;
  end else
  begin
    access := GENERIC_READ;
    creation := OPEN_EXISTING;
    if FileName = '' then
    begin
      FName := '<STDIN>';
      FHandle := GetStdHandle(STD_INPUT_HANDLE);
    end;
  end;
  if FileName <> '' then
  begin
    FName := FileName;
    FHandle := CreateFile(PChar(FName), access, sharemode, nil, creation,
                          FILE_ATTRIBUTE_NORMAL, 0);
  end;
  if FHandle = INVALID_HANDLE_VALUE then
    raise ESIOError.Create(FName);
end;

destructor _TSFile.Destroy;
begin
  if FHandle <> INVALID_HANDLE_VALUE then
    CloseHandle(FHandle);
end;

function _TSFile.GetSize: Cardinal;
begin
  Result := GetFileSize(FHandle, nil);
  if Result = $FFFFFFFF then
    raise ESIOError.Create(FName);
end;

function _TSFile.Read(var Buffer; Count: Integer): Cardinal;
begin
  if not ReadFile(FHandle, Buffer, Count, DWORD(Result), nil) then
    raise ESIOError.Create(FName);
end;

function _TSFile.Write(var Buffer; Count: Integer): Cardinal;
begin
  if not WriteFile(FHandle, Buffer, Count, DWORD(Result), nil) then
    raise ESIOError.Create(FName);
end;

function _TSFile.Seek(Offset: Integer): Cardinal;
begin
  Result := SetFilePointer(FHandle, Offset, nil, FILE_BEGIN);
  if Result = $FFFFFFFF then
    raise ESIOError.Create(FName);
end;


{ _TListDir }

constructor _TListDir.Create(const Path: string);
begin
  CreateWithGlob(Path, '*.*');
end;

constructor _TListDir.CreateWithGlob(const Path: string; Glob: string);
begin
  FHandle := FindFirstFile(PChar(Path+'\'+Glob), FFindData);
  FResult := FHandle <> INVALID_HANDLE_VALUE;
end;

destructor _TListDir.Destroy;
begin
  if FHandle <> INVALID_HANDLE_VALUE then
    FindClose(FHandle);
  inherited;
end;

function _TListDir.Each(var FileName: string): Boolean;
begin
  Result := FResult;
  if FResult then
  begin
    FileName := FFindData.cFileName;
    FIsDir := (FFindData.dwFileAttributes and FILE_ATTRIBUTE_DIRECTORY) <> 0;
    FResult := FindNextFile(FHandle, FFindData);
  end;
end;

end.

